# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_image_scan_finding import ModelImageScanFinding


class ApimodelGetAppImageListV2DataItemImageScanResult(Model):
    """Apimodel get app image list V2 data item image scan result (apimodel.GetAppImageListV2DataItemImageScanResult)

    Properties:
        finding_severity_counts: (findingSeverityCounts) REQUIRED Dict[str, int]

        findings: (findings) REQUIRED List[ModelImageScanFinding]
    """

    # region fields

    finding_severity_counts: Dict[str, int]  # REQUIRED
    findings: List[ModelImageScanFinding]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_finding_severity_counts(
        self, value: Dict[str, int]
    ) -> ApimodelGetAppImageListV2DataItemImageScanResult:
        self.finding_severity_counts = value
        return self

    def with_findings(
        self, value: List[ModelImageScanFinding]
    ) -> ApimodelGetAppImageListV2DataItemImageScanResult:
        self.findings = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "finding_severity_counts"):
            result["findingSeverityCounts"] = {
                str(k0): int(v0) for k0, v0 in self.finding_severity_counts.items()
            }
        elif include_empty:
            result["findingSeverityCounts"] = {}
        if hasattr(self, "findings"):
            result["findings"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.findings
            ]
        elif include_empty:
            result["findings"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        finding_severity_counts: Dict[str, int],
        findings: List[ModelImageScanFinding],
        **kwargs,
    ) -> ApimodelGetAppImageListV2DataItemImageScanResult:
        instance = cls()
        instance.finding_severity_counts = finding_severity_counts
        instance.findings = findings
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelGetAppImageListV2DataItemImageScanResult:
        instance = cls()
        if not dict_:
            return instance
        if (
            "findingSeverityCounts" in dict_
            and dict_["findingSeverityCounts"] is not None
        ):
            instance.finding_severity_counts = {
                str(k0): int(v0) for k0, v0 in dict_["findingSeverityCounts"].items()
            }
        elif include_empty:
            instance.finding_severity_counts = {}
        if "findings" in dict_ and dict_["findings"] is not None:
            instance.findings = [
                ModelImageScanFinding.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["findings"]
            ]
        elif include_empty:
            instance.findings = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelGetAppImageListV2DataItemImageScanResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelGetAppImageListV2DataItemImageScanResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelGetAppImageListV2DataItemImageScanResult,
        List[ApimodelGetAppImageListV2DataItemImageScanResult],
        Dict[Any, ApimodelGetAppImageListV2DataItemImageScanResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "findingSeverityCounts": "finding_severity_counts",
            "findings": "findings",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "findingSeverityCounts": True,
            "findings": True,
        }

    # endregion static methods



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from typing import Dict, List, Optional, Union

from ...core import (
    SDK,
    AccelByteSDK,
    create_pkce_verifier_and_challenge_s256,
)

from ...api import iam
from ...api.iam.operations import o_auth2_0 as oauth2

from ._login_const import (
    DEFAULT_SCOPE,
    DEFAULT_REFRESH_RATE,
    LOGIN_TYPE_USER,
)

from ._login_refresh import (
    try_refresh_login_internal,
    try_refresh_login_async_internal,
)


def login_user_internal(
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    refresh_if_possible: bool = False,
    refresh_rate: float = DEFAULT_REFRESH_RATE,
    x_additional_headers: Optional[Dict[str, str]] = None,
    sdk: Optional[AccelByteSDK] = None,
    **kwargs,
):
    sdk = sdk or SDK

    token: Optional[str] = None
    if refresh_if_possible:
        token = try_refresh_login_internal(
            refresh_rate=refresh_rate,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )

    if not token:
        (
            code_verifier,
            code_challenge,
            code_challenge_method,
        ) = create_pkce_verifier_and_challenge_s256()

        if scope is None:
            scope = DEFAULT_SCOPE
        if isinstance(scope, list):
            scope = " ".join(scope)

        client_id, error = sdk.get_client_id()
        if error:
            return None, error

        authorization_request_id, error = iam.authorize_v3(
            client_id=client_id,
            response_type=oauth2.AuthorizeV3ResponseTypeEnum.CODE,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            scope=scope,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )
        if error:
            return None, error

        authorization_code, error = iam.user_authentication_v3(
            user_name=username,
            password=password,
            request_id=authorization_request_id,
            client_id=client_id,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )
        if error:
            return None, error

        token, error = iam.token_grant_v3(
            grant_type=oauth2.TokenGrantV3GrantTypeEnum.AUTHORIZATION_CODE,
            code=authorization_code,
            code_verifier=code_verifier,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )
        if error:
            return None, error

    assert token, "Token value missing!"

    _, error = sdk.set_token(token=token, type=LOGIN_TYPE_USER)
    if error:
        return None, error

    return token, None


async def login_user_async_internal(
    username: str,
    password: str,
    scope: Optional[Union[str, List[str]]] = None,
    refresh_if_possible: bool = False,
    refresh_rate: float = DEFAULT_REFRESH_RATE,
    x_additional_headers: Optional[Dict[str, str]] = None,
    sdk: Optional[AccelByteSDK] = None,
    **kwargs,
):
    sdk = sdk or SDK

    token: Optional[str] = None
    if refresh_if_possible:
        token = await try_refresh_login_async_internal(
            refresh_rate=refresh_rate,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )

    if not token:
        (
            code_verifier,
            code_challenge,
            code_challenge_method,
        ) = create_pkce_verifier_and_challenge_s256()

        if scope is None:
            scope = DEFAULT_SCOPE
        if isinstance(scope, list):
            scope = " ".join(scope)

        client_id, error = sdk.get_client_id()
        if error:
            return None, error

        authorization_request_id, error = await iam.authorize_v3_async(
            client_id=client_id,
            response_type=oauth2.AuthorizeV3ResponseTypeEnum.CODE,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            scope=scope,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )
        if error:
            return None, error

        authorization_code, error = await iam.user_authentication_v3_async(
            user_name=username,
            password=password,
            request_id=authorization_request_id,
            client_id=client_id,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )
        if error:
            return None, error

        token, error = await iam.token_grant_v3_async(
            grant_type=oauth2.TokenGrantV3GrantTypeEnum.AUTHORIZATION_CODE,
            code=authorization_code,
            code_verifier=code_verifier,
            x_additional_headers=x_additional_headers,
            sdk=sdk,
            **kwargs,
        )
        if error:
            return None, error

    assert token, "Token value missing!"

    _, error = sdk.set_token(token=token, type=LOGIN_TYPE_USER)
    if error:
        return None, error

    return token, None


__all__ = [
    "login_user_internal",
    "login_user_async_internal",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelInternalConfigResponseV3(Model):
    """Model internal config response V3 (model.InternalConfigResponseV3)

    Properties:
        search_query_max_length: (searchQueryMaxLength) REQUIRED int

        search_query_max_length_for_email: (searchQueryMaxLengthForEmail) REQUIRED int

        search_query_min_length: (searchQueryMinLength) REQUIRED int
    """

    # region fields

    search_query_max_length: int  # REQUIRED
    search_query_max_length_for_email: int  # REQUIRED
    search_query_min_length: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_search_query_max_length(self, value: int) -> ModelInternalConfigResponseV3:
        self.search_query_max_length = value
        return self

    def with_search_query_max_length_for_email(
        self, value: int
    ) -> ModelInternalConfigResponseV3:
        self.search_query_max_length_for_email = value
        return self

    def with_search_query_min_length(self, value: int) -> ModelInternalConfigResponseV3:
        self.search_query_min_length = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "search_query_max_length"):
            result["searchQueryMaxLength"] = int(self.search_query_max_length)
        elif include_empty:
            result["searchQueryMaxLength"] = 0
        if hasattr(self, "search_query_max_length_for_email"):
            result["searchQueryMaxLengthForEmail"] = int(
                self.search_query_max_length_for_email
            )
        elif include_empty:
            result["searchQueryMaxLengthForEmail"] = 0
        if hasattr(self, "search_query_min_length"):
            result["searchQueryMinLength"] = int(self.search_query_min_length)
        elif include_empty:
            result["searchQueryMinLength"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        search_query_max_length: int,
        search_query_max_length_for_email: int,
        search_query_min_length: int,
        **kwargs,
    ) -> ModelInternalConfigResponseV3:
        instance = cls()
        instance.search_query_max_length = search_query_max_length
        instance.search_query_max_length_for_email = search_query_max_length_for_email
        instance.search_query_min_length = search_query_min_length
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelInternalConfigResponseV3:
        instance = cls()
        if not dict_:
            return instance
        if (
            "searchQueryMaxLength" in dict_
            and dict_["searchQueryMaxLength"] is not None
        ):
            instance.search_query_max_length = int(dict_["searchQueryMaxLength"])
        elif include_empty:
            instance.search_query_max_length = 0
        if (
            "searchQueryMaxLengthForEmail" in dict_
            and dict_["searchQueryMaxLengthForEmail"] is not None
        ):
            instance.search_query_max_length_for_email = int(
                dict_["searchQueryMaxLengthForEmail"]
            )
        elif include_empty:
            instance.search_query_max_length_for_email = 0
        if (
            "searchQueryMinLength" in dict_
            and dict_["searchQueryMinLength"] is not None
        ):
            instance.search_query_min_length = int(dict_["searchQueryMinLength"])
        elif include_empty:
            instance.search_query_min_length = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelInternalConfigResponseV3]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelInternalConfigResponseV3]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelInternalConfigResponseV3,
        List[ModelInternalConfigResponseV3],
        Dict[Any, ModelInternalConfigResponseV3],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "searchQueryMaxLength": "search_query_max_length",
            "searchQueryMaxLengthForEmail": "search_query_max_length_for_email",
            "searchQueryMinLength": "search_query_min_length",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "searchQueryMaxLength": True,
            "searchQueryMaxLengthForEmail": True,
            "searchQueryMinLength": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import GeneratedGetListOfConfigurationsV1Response
from ..models import GeneratedSaveConfigurationV1Request
from ..models import GeneratedSaveConfigurationV1Response
from ..models import GeneratedUpdateConfigurationV1Request
from ..models import GeneratedUpdateConfigurationV1Response
from ..models import ResponseErrorResponse

from ..operations.configuration import DeleteSecretV1
from ..operations.configuration import DeleteVariableV1
from ..operations.configuration import GetListOfSecretsV1
from ..operations.configuration import GetListOfVariablesV1
from ..operations.configuration import SaveSecretV1
from ..operations.configuration import SaveVariableV1
from ..operations.configuration import UpdateSecretV1
from ..operations.configuration import UpdateVariableV1


@deprecated
@same_doc_as(DeleteSecretV1)
def delete_secret_v1(
    app: str,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete an environment secret (DeleteSecretV1)

    Delete an environment secret.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets/{configId}

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteSecretV1.create(
        app=app,
        config_id=config_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteSecretV1)
async def delete_secret_v1_async(
    app: str,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete an environment secret (DeleteSecretV1)

    Delete an environment secret.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets/{configId}

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteSecretV1.create(
        app=app,
        config_id=config_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(DeleteVariableV1)
def delete_variable_v1(
    app: str,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete an environment variable (DeleteVariableV1)

    Delete an environment variable.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables/{configId}

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteVariableV1.create(
        app=app,
        config_id=config_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteVariableV1)
async def delete_variable_v1_async(
    app: str,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete an environment variable (DeleteVariableV1)

    Delete an environment variable.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables/{configId}

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteVariableV1.create(
        app=app,
        config_id=config_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetListOfSecretsV1)
def get_list_of_secrets_v1(
    app: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of environment secrets (GetListOfSecretsV1)

    Get list of environment secrets per app
    Available Deployment Status:
    `deployed` = app config is already deployed
    `undeployed` = app config is not deployed yet and need restart and deploy of the app to be deployed

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - GeneratedGetListOfConfigurationsV1Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetListOfSecretsV1.create(
        app=app,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetListOfSecretsV1)
async def get_list_of_secrets_v1_async(
    app: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of environment secrets (GetListOfSecretsV1)

    Get list of environment secrets per app
    Available Deployment Status:
    `deployed` = app config is already deployed
    `undeployed` = app config is not deployed yet and need restart and deploy of the app to be deployed

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - GeneratedGetListOfConfigurationsV1Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetListOfSecretsV1.create(
        app=app,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetListOfVariablesV1)
def get_list_of_variables_v1(
    app: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of environment variables (GetListOfVariablesV1)

    Get list of environment variables per app
    Available Deployment Status:
    `deployed` = app config is already deployed
    `undeployed` = app config is not deployed yet and need restart and deploy of the app to be deployed

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - GeneratedGetListOfConfigurationsV1Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetListOfVariablesV1.create(
        app=app,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetListOfVariablesV1)
async def get_list_of_variables_v1_async(
    app: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get list of environment variables (GetListOfVariablesV1)

    Get list of environment variables per app
    Available Deployment Status:
    `deployed` = app config is already deployed
    `undeployed` = app config is not deployed yet and need restart and deploy of the app to be deployed

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - GeneratedGetListOfConfigurationsV1Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetListOfVariablesV1.create(
        app=app,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(SaveSecretV1)
def save_secret_v1(
    app: str,
    body: GeneratedSaveConfigurationV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Save an environment secret (SaveSecretV1)

    Save an environment secret.
    Request body:
    - configName : environment secret name - Required.
    - source : source of the configuration value (plaintext or ssm) - Required.
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets

        method: POST

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedSaveConfigurationV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedSaveConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = SaveSecretV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(SaveSecretV1)
async def save_secret_v1_async(
    app: str,
    body: GeneratedSaveConfigurationV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Save an environment secret (SaveSecretV1)

    Save an environment secret.
    Request body:
    - configName : environment secret name - Required.
    - source : source of the configuration value (plaintext or ssm) - Required.
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets

        method: POST

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedSaveConfigurationV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedSaveConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = SaveSecretV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(SaveVariableV1)
def save_variable_v1(
    app: str,
    body: GeneratedSaveConfigurationV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Save an environment variable (SaveVariableV1)

    Save an environment variable.
    Request body:
    - configName : environment variable name - Required.
    - source : source of the configuration value (plaintext or ssm) - Required.
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables

        method: POST

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedSaveConfigurationV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedSaveConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = SaveVariableV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(SaveVariableV1)
async def save_variable_v1_async(
    app: str,
    body: GeneratedSaveConfigurationV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Save an environment variable (SaveVariableV1)

    Save an environment variable.
    Request body:
    - configName : environment variable name - Required.
    - source : source of the configuration value (plaintext or ssm) - Required.
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables

        method: POST

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedSaveConfigurationV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedSaveConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = SaveVariableV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(UpdateSecretV1)
def update_secret_v1(
    app: str,
    body: GeneratedUpdateConfigurationV1Request,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update an environment secret (UpdateSecretV1)

    Update an environment secret.
    Request body:
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets/{configId}

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedUpdateConfigurationV1Request in body

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedUpdateConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateSecretV1.create(
        app=app,
        body=body,
        config_id=config_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(UpdateSecretV1)
async def update_secret_v1_async(
    app: str,
    body: GeneratedUpdateConfigurationV1Request,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update an environment secret (UpdateSecretV1)

    Update an environment secret.
    Request body:
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/secrets/{configId}

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedUpdateConfigurationV1Request in body

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedUpdateConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateSecretV1.create(
        app=app,
        body=body,
        config_id=config_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(UpdateVariableV1)
def update_variable_v1(
    app: str,
    body: GeneratedUpdateConfigurationV1Request,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update an environment variable (UpdateVariableV1)

    Update an environment variable.
    Request body:
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables/{configId}

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedUpdateConfigurationV1Request in body

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedUpdateConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateVariableV1.create(
        app=app,
        body=body,
        config_id=config_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(UpdateVariableV1)
async def update_variable_v1_async(
    app: str,
    body: GeneratedUpdateConfigurationV1Request,
    config_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update an environment variable (UpdateVariableV1)

    Update an environment variable.
    Request body:
    - value : configuration value - Required.
    - description : description of the configuration - Optional.
    - applyMask : mask the value in the Helm manifest for sensitive information (true or false) - Optional.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/variables/{configId}

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedUpdateConfigurationV1Request in body

        app: (app) REQUIRED str in path

        config_id: (configId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedUpdateConfigurationV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdateVariableV1.create(
        app=app,
        body=body,
        config_id=config_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



from typing import Optional, Protocol

from ._ctypes import (
    FetchValidationDataError,
    FetchNamespaceContextError,
    FetchRoleError,
    TokenValidationError,
    InsufficientPermissionsError,
    TokenRevokedError,
    UserRevokedError,
    PermissionAction,
    PermissionActionFlag,
    PermissionStruct,
    create_permission_struct,
)
from ._utils import replace_resource
from ._validation import (
    validate_action,
    validate_permission,
    validate_resource,
)


class TokenValidatorProtocol(Protocol):
    def validate_token(
        self,
        token: str,
        resource: Optional[str] = None,
        action: Optional[PermissionAction] = None,
        namespace: Optional[str] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> Optional[Exception]:
        ...


__all__ = [
    "FetchValidationDataError",
    "FetchNamespaceContextError",
    "FetchRoleError",
    "TokenValidationError",
    "InsufficientPermissionsError",
    "TokenRevokedError",
    "UserRevokedError",
    "PermissionAction",
    "PermissionActionFlag",
    "PermissionStruct",
    "TokenValidatorProtocol",
    "create_permission_struct",
    "replace_resource",
    "validate_action",
    "validate_permission",
    "validate_resource",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Session Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ResponseError


class PublicUpdateInsertPartySessionStorageReserved(Operation):
    """Update Insert Party Session Reserved Storage User. (publicUpdateInsertPartySessionStorageReserved)

    **For Internal Use Only**
    Update Insert Party Session Reserved Storage User. User can only update or insert their own reserve storage (non-immutable).
    can store generic json
    example json can store :
    ```
    {
    "key": "value",
    "number": 123,
    }
    ```
    The data will be stored on the "reserved" storage field
    example stored data :
    ```
    {
    "reserved": {
    "userID1": {"key": "value"},
    "userID2": {"key": "value"},
    ...
    }
    }
    ```

    Properties:
        url: /session/v1/public/namespaces/{namespace}/parties/{partyId}/storage/users/{userId}/reserved

        method: PATCH

        tags: ["Session Storage"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED Dict[str, Any] in body

        namespace: (namespace) REQUIRED str in path

        party_id: (partyId) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - Dict[str, Any] (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """

    # region fields

    _url: str = "/session/v1/public/namespaces/{namespace}/parties/{partyId}/storage/users/{userId}/reserved"
    _method: str = "PATCH"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    body: Dict[str, Any]  # REQUIRED in [body]
    namespace: str  # REQUIRED in [path]
    party_id: str  # REQUIRED in [path]
    user_id: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        if hasattr(self, "party_id"):
            result["partyId"] = self.party_id
        if hasattr(self, "user_id"):
            result["userId"] = self.user_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(
        self, value: Dict[str, Any]
    ) -> PublicUpdateInsertPartySessionStorageReserved:
        self.body = value
        return self

    def with_namespace(
        self, value: str
    ) -> PublicUpdateInsertPartySessionStorageReserved:
        self.namespace = value
        return self

    def with_party_id(
        self, value: str
    ) -> PublicUpdateInsertPartySessionStorageReserved:
        self.party_id = value
        return self

    def with_user_id(self, value: str) -> PublicUpdateInsertPartySessionStorageReserved:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = {str(k0): v0 for k0, v0 in self.body.items()}
        elif include_empty:
            result["body"] = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "party_id") and self.party_id:
            result["partyId"] = str(self.party_id)
        elif include_empty:
            result["partyId"] = ""
        if hasattr(self, "user_id") and self.user_id:
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[Union[None, Dict[str, Any]], Union[None, HttpResponse, ResponseError]]:
        """Parse the given response.

        200: OK - Dict[str, Any] (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return {str(k): v for k, v in content.items()}, None
        if code == 400:
            return None, ResponseError.create_from_dict(content)
        if code == 401:
            return None, ResponseError.create_from_dict(content)
        if code == 403:
            return None, ResponseError.create_from_dict(content)
        if code == 404:
            return None, ResponseError.create_from_dict(content)
        if code == 500:
            return None, ResponseError.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, body: Dict[str, Any], namespace: str, party_id: str, user_id: str, **kwargs
    ) -> PublicUpdateInsertPartySessionStorageReserved:
        instance = cls()
        instance.body = body
        instance.namespace = namespace
        instance.party_id = party_id
        instance.user_id = user_id
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicUpdateInsertPartySessionStorageReserved:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = {str(k0): v0 for k0, v0 in dict_["body"].items()}
        elif include_empty:
            instance.body = {}
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "partyId" in dict_ and dict_["partyId"] is not None:
            instance.party_id = str(dict_["partyId"])
        elif include_empty:
            instance.party_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
            "partyId": "party_id",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
            "namespace": True,
            "partyId": True,
            "userId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Challenge Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_goal_order import ModelGoalOrder


class ModelSchedule(Model):
    """Model schedule (model.Schedule)

    Properties:
        challenge_code: (challengeCode) REQUIRED str

        created_at: (createdAt) REQUIRED str

        end_time: (endTime) REQUIRED str

        goals: (goals) REQUIRED List[ModelGoalOrder]

        namespace: (namespace) REQUIRED str

        start_time: (startTime) REQUIRED str

        updated_at: (updatedAt) REQUIRED str
    """

    # region fields

    challenge_code: str  # REQUIRED
    created_at: str  # REQUIRED
    end_time: str  # REQUIRED
    goals: List[ModelGoalOrder]  # REQUIRED
    namespace: str  # REQUIRED
    start_time: str  # REQUIRED
    updated_at: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_challenge_code(self, value: str) -> ModelSchedule:
        self.challenge_code = value
        return self

    def with_created_at(self, value: str) -> ModelSchedule:
        self.created_at = value
        return self

    def with_end_time(self, value: str) -> ModelSchedule:
        self.end_time = value
        return self

    def with_goals(self, value: List[ModelGoalOrder]) -> ModelSchedule:
        self.goals = value
        return self

    def with_namespace(self, value: str) -> ModelSchedule:
        self.namespace = value
        return self

    def with_start_time(self, value: str) -> ModelSchedule:
        self.start_time = value
        return self

    def with_updated_at(self, value: str) -> ModelSchedule:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "challenge_code"):
            result["challengeCode"] = str(self.challenge_code)
        elif include_empty:
            result["challengeCode"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "end_time"):
            result["endTime"] = str(self.end_time)
        elif include_empty:
            result["endTime"] = ""
        if hasattr(self, "goals"):
            result["goals"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.goals
            ]
        elif include_empty:
            result["goals"] = []
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "start_time"):
            result["startTime"] = str(self.start_time)
        elif include_empty:
            result["startTime"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        challenge_code: str,
        created_at: str,
        end_time: str,
        goals: List[ModelGoalOrder],
        namespace: str,
        start_time: str,
        updated_at: str,
        **kwargs,
    ) -> ModelSchedule:
        instance = cls()
        instance.challenge_code = challenge_code
        instance.created_at = created_at
        instance.end_time = end_time
        instance.goals = goals
        instance.namespace = namespace
        instance.start_time = start_time
        instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelSchedule:
        instance = cls()
        if not dict_:
            return instance
        if "challengeCode" in dict_ and dict_["challengeCode"] is not None:
            instance.challenge_code = str(dict_["challengeCode"])
        elif include_empty:
            instance.challenge_code = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "endTime" in dict_ and dict_["endTime"] is not None:
            instance.end_time = str(dict_["endTime"])
        elif include_empty:
            instance.end_time = ""
        if "goals" in dict_ and dict_["goals"] is not None:
            instance.goals = [
                ModelGoalOrder.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["goals"]
            ]
        elif include_empty:
            instance.goals = []
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "startTime" in dict_ and dict_["startTime"] is not None:
            instance.start_time = str(dict_["startTime"])
        elif include_empty:
            instance.start_time = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelSchedule]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelSchedule]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelSchedule, List[ModelSchedule], Dict[Any, ModelSchedule]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "challengeCode": "challenge_code",
            "createdAt": "created_at",
            "endTime": "end_time",
            "goals": "goals",
            "namespace": "namespace",
            "startTime": "start_time",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "challengeCode": True,
            "createdAt": True,
            "endTime": True,
            "goals": True,
            "namespace": True,
            "startTime": True,
            "updatedAt": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Match Service V2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.matchmaker_proposed_proposal import MatchmakerProposedProposal
from ..models.matchmaker_ticket import MatchmakerTicket


class MatchmakerMatchTicketRecord(Model):
    """Matchmaker match ticket record (matchmaker.MatchTicketRecord)

    Properties:
        created_at: (CreatedAt) REQUIRED str

        expired_at: (ExpiredAt) REQUIRED str

        is_active: (IsActive) REQUIRED bool

        party_session_id: (PartySessionID) REQUIRED str

        proposed_proposal: (ProposedProposal) REQUIRED MatchmakerProposedProposal

        session_id: (SessionID) REQUIRED str

        ticket: (Ticket) REQUIRED MatchmakerTicket

        ticket_id: (TicketID) REQUIRED str

        unique_ticket_id: (UniqueTicketID) REQUIRED str
    """

    # region fields

    created_at: str  # REQUIRED
    expired_at: str  # REQUIRED
    is_active: bool  # REQUIRED
    party_session_id: str  # REQUIRED
    proposed_proposal: MatchmakerProposedProposal  # REQUIRED
    session_id: str  # REQUIRED
    ticket: MatchmakerTicket  # REQUIRED
    ticket_id: str  # REQUIRED
    unique_ticket_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> MatchmakerMatchTicketRecord:
        self.created_at = value
        return self

    def with_expired_at(self, value: str) -> MatchmakerMatchTicketRecord:
        self.expired_at = value
        return self

    def with_is_active(self, value: bool) -> MatchmakerMatchTicketRecord:
        self.is_active = value
        return self

    def with_party_session_id(self, value: str) -> MatchmakerMatchTicketRecord:
        self.party_session_id = value
        return self

    def with_proposed_proposal(
        self, value: MatchmakerProposedProposal
    ) -> MatchmakerMatchTicketRecord:
        self.proposed_proposal = value
        return self

    def with_session_id(self, value: str) -> MatchmakerMatchTicketRecord:
        self.session_id = value
        return self

    def with_ticket(self, value: MatchmakerTicket) -> MatchmakerMatchTicketRecord:
        self.ticket = value
        return self

    def with_ticket_id(self, value: str) -> MatchmakerMatchTicketRecord:
        self.ticket_id = value
        return self

    def with_unique_ticket_id(self, value: str) -> MatchmakerMatchTicketRecord:
        self.unique_ticket_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["CreatedAt"] = str(self.created_at)
        elif include_empty:
            result["CreatedAt"] = ""
        if hasattr(self, "expired_at"):
            result["ExpiredAt"] = str(self.expired_at)
        elif include_empty:
            result["ExpiredAt"] = ""
        if hasattr(self, "is_active"):
            result["IsActive"] = bool(self.is_active)
        elif include_empty:
            result["IsActive"] = False
        if hasattr(self, "party_session_id"):
            result["PartySessionID"] = str(self.party_session_id)
        elif include_empty:
            result["PartySessionID"] = ""
        if hasattr(self, "proposed_proposal"):
            result["ProposedProposal"] = self.proposed_proposal.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["ProposedProposal"] = MatchmakerProposedProposal()
        if hasattr(self, "session_id"):
            result["SessionID"] = str(self.session_id)
        elif include_empty:
            result["SessionID"] = ""
        if hasattr(self, "ticket"):
            result["Ticket"] = self.ticket.to_dict(include_empty=include_empty)
        elif include_empty:
            result["Ticket"] = MatchmakerTicket()
        if hasattr(self, "ticket_id"):
            result["TicketID"] = str(self.ticket_id)
        elif include_empty:
            result["TicketID"] = ""
        if hasattr(self, "unique_ticket_id"):
            result["UniqueTicketID"] = str(self.unique_ticket_id)
        elif include_empty:
            result["UniqueTicketID"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        expired_at: str,
        is_active: bool,
        party_session_id: str,
        proposed_proposal: MatchmakerProposedProposal,
        session_id: str,
        ticket: MatchmakerTicket,
        ticket_id: str,
        unique_ticket_id: str,
        **kwargs,
    ) -> MatchmakerMatchTicketRecord:
        instance = cls()
        instance.created_at = created_at
        instance.expired_at = expired_at
        instance.is_active = is_active
        instance.party_session_id = party_session_id
        instance.proposed_proposal = proposed_proposal
        instance.session_id = session_id
        instance.ticket = ticket
        instance.ticket_id = ticket_id
        instance.unique_ticket_id = unique_ticket_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> MatchmakerMatchTicketRecord:
        instance = cls()
        if not dict_:
            return instance
        if "CreatedAt" in dict_ and dict_["CreatedAt"] is not None:
            instance.created_at = str(dict_["CreatedAt"])
        elif include_empty:
            instance.created_at = ""
        if "ExpiredAt" in dict_ and dict_["ExpiredAt"] is not None:
            instance.expired_at = str(dict_["ExpiredAt"])
        elif include_empty:
            instance.expired_at = ""
        if "IsActive" in dict_ and dict_["IsActive"] is not None:
            instance.is_active = bool(dict_["IsActive"])
        elif include_empty:
            instance.is_active = False
        if "PartySessionID" in dict_ and dict_["PartySessionID"] is not None:
            instance.party_session_id = str(dict_["PartySessionID"])
        elif include_empty:
            instance.party_session_id = ""
        if "ProposedProposal" in dict_ and dict_["ProposedProposal"] is not None:
            instance.proposed_proposal = MatchmakerProposedProposal.create_from_dict(
                dict_["ProposedProposal"], include_empty=include_empty
            )
        elif include_empty:
            instance.proposed_proposal = MatchmakerProposedProposal()
        if "SessionID" in dict_ and dict_["SessionID"] is not None:
            instance.session_id = str(dict_["SessionID"])
        elif include_empty:
            instance.session_id = ""
        if "Ticket" in dict_ and dict_["Ticket"] is not None:
            instance.ticket = MatchmakerTicket.create_from_dict(
                dict_["Ticket"], include_empty=include_empty
            )
        elif include_empty:
            instance.ticket = MatchmakerTicket()
        if "TicketID" in dict_ and dict_["TicketID"] is not None:
            instance.ticket_id = str(dict_["TicketID"])
        elif include_empty:
            instance.ticket_id = ""
        if "UniqueTicketID" in dict_ and dict_["UniqueTicketID"] is not None:
            instance.unique_ticket_id = str(dict_["UniqueTicketID"])
        elif include_empty:
            instance.unique_ticket_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, MatchmakerMatchTicketRecord]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[MatchmakerMatchTicketRecord]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        MatchmakerMatchTicketRecord,
        List[MatchmakerMatchTicketRecord],
        Dict[Any, MatchmakerMatchTicketRecord],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "CreatedAt": "created_at",
            "ExpiredAt": "expired_at",
            "IsActive": "is_active",
            "PartySessionID": "party_session_id",
            "ProposedProposal": "proposed_proposal",
            "SessionID": "session_id",
            "Ticket": "ticket",
            "TicketID": "ticket_id",
            "UniqueTicketID": "unique_ticket_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "CreatedAt": True,
            "ExpiredAt": True,
            "IsActive": True,
            "PartySessionID": True,
            "ProposedProposal": True,
            "SessionID": True,
            "Ticket": True,
            "TicketID": True,
            "UniqueTicketID": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from __future__ import annotations

from abc import ABC, abstractmethod
from datetime import datetime, timedelta
from typing import Any, List, Optional
from typing import Protocol, runtime_checkable

from ._utils import get_member
from ._utils import clamp


@runtime_checkable
class TokenRepositoryObserver(Protocol):
    def on_access_token_changed(self, access_token: Optional[str]) -> None:
        ...


class TokenRepository(ABC):
    @abstractmethod
    def get_token(self) -> Any:
        pass

    @abstractmethod
    def get_token_issued_time_utc(self) -> Optional[datetime]:
        pass

    @abstractmethod
    def remove_token(self) -> bool:
        pass

    @abstractmethod
    def store_token(self, token: Any, **kwargs) -> bool:
        pass

    def get_access_token(self, default: Any = None) -> Any:
        return get_member(self.get_token(), "access_token", default=default)

    def get_expires_in(self, default: Any = None) -> Any:
        return get_member(self.get_token(), "expires_in", default=default)

    def get_refresh_expires_in(self, default: Any = None) -> Any:
        return get_member(self.get_token(), "refresh_expires_in", default=default)

    def get_refresh_token(self, default: Any = None) -> Any:
        return get_member(self.get_token(), "refresh_token", default=default)

    def get_seconds_till_expiry(self) -> float:
        if not self.has_token():
            return 0
        if not (token_issued_time := self.get_token_issued_time_utc()):
            return 0
        if not (expires_in := self.get_expires_in()):
            return 0
        now = datetime.utcnow()
        expires_at = token_issued_time + timedelta(seconds=expires_in)
        seconds_till_expiry = (expires_at - now).total_seconds()
        return seconds_till_expiry

    def has_token(self) -> bool:
        return self.get_token() is not None

    def has_token_expired(self, multiplier: float = 0.0) -> bool:
        if not self.has_token():
            return False
        expires_in = self.get_expires_in()
        if expires_in is None:
            return False
        threshold = expires_in * (1.0 - clamp(multiplier, 0.0, 1.0))
        seconds_till_expiry = self.get_seconds_till_expiry()
        has_token_expired = seconds_till_expiry <= threshold
        return has_token_expired

    # noinspection PyMethodMayBeStatic
    def get_token_type(self) -> Optional[str]:
        return None

    # noinspection PyMethodMayBeStatic
    def register_observer(self, observer: TokenRepositoryObserver) -> bool:
        return True

    # noinspection PyMethodMayBeStatic
    def unregister_observer(self, observer: TokenRepositoryObserver) -> bool:
        return True


class MyTokenRepository(TokenRepository):
    def __init__(self, token: Any, **kwargs):
        self._token: Any = None
        self._token_issued_time: Optional[datetime] = None
        self._token_type: Optional[str] = None
        self._observers: List[TokenRepositoryObserver] = []

        self.store_token(token, **kwargs)

    def get_token(self) -> Any:
        return self._token

    def get_token_issued_time_utc(self) -> Optional[datetime]:
        return self._token_issued_time

    def remove_token(self) -> bool:
        if self._token is not None:
            self._token = None
            self._token_issued_time = None
            self._token_type = None
            access_token = self.get_access_token()
            for observer in self._observers:
                observer.on_access_token_changed(access_token)
            return True
        return True

    def store_token(self, token: Any, **kwargs) -> bool:
        self._token = token
        self._token_issued_time = datetime.utcnow()
        self._token_type = str(kwargs["type"]).lower() if "type" in kwargs else None
        access_token = self.get_access_token()
        for observer in self._observers:
            observer.on_access_token_changed(access_token)
        return True

    def get_token_type(self) -> Optional[str]:
        return self._token_type

    def register_observer(self, observer: TokenRepositoryObserver) -> bool:
        if observer in self._observers:
            return False
        self._observers.append(observer)
        return True

    def unregister_observer(self, observer: TokenRepositoryObserver) -> bool:
        if observer not in self._observers:
            return False
        self._observers.remove(observer)
        return True


class InMemoryTokenRepository(TokenRepository):
    def __init__(self):
        self._token: Any = None
        self._token_issued_time: Optional[datetime] = None
        self._token_type: Optional[str] = None
        self._observers: List[TokenRepositoryObserver] = []

    def get_token(self) -> Any:
        return self._token

    def get_token_issued_time_utc(self) -> Optional[datetime]:
        return self._token_issued_time

    def remove_token(self) -> bool:
        if self._token is not None:
            self._token = None
            self._token_issued_time = None
            self._token_type = None
            access_token = self.get_access_token()
            for observer in self._observers:
                observer.on_access_token_changed(access_token)
            return True
        return True

    def store_token(self, token: Any, **kwargs) -> bool:
        self._token = token
        self._token_issued_time = datetime.utcnow()
        self._token_type = str(kwargs["type"]).lower() if "type" in kwargs else None
        access_token = self.get_access_token()
        for observer in self._observers:
            observer.on_access_token_changed(access_token)
        return True

    def get_token_type(self) -> Optional[str]:
        return self._token_type

    def register_observer(self, observer: TokenRepositoryObserver) -> bool:
        if observer in self._observers:
            return False
        self._observers.append(observer)
        return True

    def unregister_observer(self, observer: TokenRepositoryObserver) -> bool:
        if observer not in self._observers:
            return False
        self._observers.remove(observer)
        return True


TOKEN_REPOS = [MyTokenRepository, InMemoryTokenRepository]

DEFAULT_TOKEN_REPO = InMemoryTokenRepository



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Gdpr Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class DtoPlatformAccountClosureClientRequest(Model):
    """Dto platform account closure client request (dto.PlatformAccountClosureClientRequest)

    Properties:
        bp_cert: (bpCert) OPTIONAL str

        bp_cert_file_name: (bpCertFileName) OPTIONAL str

        client_id: (clientId) OPTIONAL str

        password: (password) OPTIONAL str

        publisher_key: (publisherKey) OPTIONAL str

        sandbox_id: (sandboxId) OPTIONAL str

        secret: (secret) OPTIONAL str
    """

    # region fields

    bp_cert: str  # OPTIONAL
    bp_cert_file_name: str  # OPTIONAL
    client_id: str  # OPTIONAL
    password: str  # OPTIONAL
    publisher_key: str  # OPTIONAL
    sandbox_id: str  # OPTIONAL
    secret: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_bp_cert(self, value: str) -> DtoPlatformAccountClosureClientRequest:
        self.bp_cert = value
        return self

    def with_bp_cert_file_name(
        self, value: str
    ) -> DtoPlatformAccountClosureClientRequest:
        self.bp_cert_file_name = value
        return self

    def with_client_id(self, value: str) -> DtoPlatformAccountClosureClientRequest:
        self.client_id = value
        return self

    def with_password(self, value: str) -> DtoPlatformAccountClosureClientRequest:
        self.password = value
        return self

    def with_publisher_key(self, value: str) -> DtoPlatformAccountClosureClientRequest:
        self.publisher_key = value
        return self

    def with_sandbox_id(self, value: str) -> DtoPlatformAccountClosureClientRequest:
        self.sandbox_id = value
        return self

    def with_secret(self, value: str) -> DtoPlatformAccountClosureClientRequest:
        self.secret = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "bp_cert"):
            result["bpCert"] = str(self.bp_cert)
        elif include_empty:
            result["bpCert"] = ""
        if hasattr(self, "bp_cert_file_name"):
            result["bpCertFileName"] = str(self.bp_cert_file_name)
        elif include_empty:
            result["bpCertFileName"] = ""
        if hasattr(self, "client_id"):
            result["clientId"] = str(self.client_id)
        elif include_empty:
            result["clientId"] = ""
        if hasattr(self, "password"):
            result["password"] = str(self.password)
        elif include_empty:
            result["password"] = ""
        if hasattr(self, "publisher_key"):
            result["publisherKey"] = str(self.publisher_key)
        elif include_empty:
            result["publisherKey"] = ""
        if hasattr(self, "sandbox_id"):
            result["sandboxId"] = str(self.sandbox_id)
        elif include_empty:
            result["sandboxId"] = ""
        if hasattr(self, "secret"):
            result["secret"] = str(self.secret)
        elif include_empty:
            result["secret"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        bp_cert: Optional[str] = None,
        bp_cert_file_name: Optional[str] = None,
        client_id: Optional[str] = None,
        password: Optional[str] = None,
        publisher_key: Optional[str] = None,
        sandbox_id: Optional[str] = None,
        secret: Optional[str] = None,
        **kwargs,
    ) -> DtoPlatformAccountClosureClientRequest:
        instance = cls()
        if bp_cert is not None:
            instance.bp_cert = bp_cert
        if bp_cert_file_name is not None:
            instance.bp_cert_file_name = bp_cert_file_name
        if client_id is not None:
            instance.client_id = client_id
        if password is not None:
            instance.password = password
        if publisher_key is not None:
            instance.publisher_key = publisher_key
        if sandbox_id is not None:
            instance.sandbox_id = sandbox_id
        if secret is not None:
            instance.secret = secret
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DtoPlatformAccountClosureClientRequest:
        instance = cls()
        if not dict_:
            return instance
        if "bpCert" in dict_ and dict_["bpCert"] is not None:
            instance.bp_cert = str(dict_["bpCert"])
        elif include_empty:
            instance.bp_cert = ""
        if "bpCertFileName" in dict_ and dict_["bpCertFileName"] is not None:
            instance.bp_cert_file_name = str(dict_["bpCertFileName"])
        elif include_empty:
            instance.bp_cert_file_name = ""
        if "clientId" in dict_ and dict_["clientId"] is not None:
            instance.client_id = str(dict_["clientId"])
        elif include_empty:
            instance.client_id = ""
        if "password" in dict_ and dict_["password"] is not None:
            instance.password = str(dict_["password"])
        elif include_empty:
            instance.password = ""
        if "publisherKey" in dict_ and dict_["publisherKey"] is not None:
            instance.publisher_key = str(dict_["publisherKey"])
        elif include_empty:
            instance.publisher_key = ""
        if "sandboxId" in dict_ and dict_["sandboxId"] is not None:
            instance.sandbox_id = str(dict_["sandboxId"])
        elif include_empty:
            instance.sandbox_id = ""
        if "secret" in dict_ and dict_["secret"] is not None:
            instance.secret = str(dict_["secret"])
        elif include_empty:
            instance.secret = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DtoPlatformAccountClosureClientRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DtoPlatformAccountClosureClientRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        DtoPlatformAccountClosureClientRequest,
        List[DtoPlatformAccountClosureClientRequest],
        Dict[Any, DtoPlatformAccountClosureClientRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "bpCert": "bp_cert",
            "bpCertFileName": "bp_cert_file_name",
            "clientId": "client_id",
            "password": "password",
            "publisherKey": "publisher_key",
            "sandboxId": "sandbox_id",
            "secret": "secret",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "bpCert": False,
            "bpCertFileName": False,
            "clientId": False,
            "password": False,
            "publisherKey": False,
            "sandboxId": False,
            "secret": False,
        }

    # endregion static methods
